import React, { HTMLProps, useEffect, useMemo, useState } from "react";
import { Button, Dialog, DialogActions, DialogContent, DialogContentText, Grid, TextField, Typography } from "@material-ui/core";
import { KeyboardDatePicker as DatePicker } from "@material-ui/pickers";
import { CompanyForm } from "@/forms/company";
import { StudentForm } from "@/forms/student";
import { sampleStudent } from "@/provider/dummy/student";
import { BranchOffice, Company, Internship, InternshipType, internshipTypeLabels, Student } from "@/data";
import { Nullable } from "@/helpers";
import moment, { Moment } from "moment";
import { computeWorkingHours } from "@/utils/date";
import { Autocomplete } from "@material-ui/lab";
import { formFieldProps } from "@/forms/helpers";
import { emptyInternship } from "@/provider/dummy/internship";
import { InternshipProposalActions, useDispatch } from "@/state/actions";
import { useTranslation } from "react-i18next";
import { useSelector } from "react-redux";
import { AppState } from "@/state/reducer";
import { Link as RouterLink, useHistory } from "react-router-dom";
import { route } from "@/routing";
import { useProxyState } from "@/hooks";
import { getInternshipProposal } from "@/state/reducer/proposal";
import { Actions } from "@/components";
import { Form, Formik } from "formik";
import * as Yup from "yup";

export type InternshipFormProps = {}

export type InternshipFormSectionProps = {
    internship: Nullable<Internship>,
    onChange: (internship: Nullable<Internship>) => void,
}

export type InternshipFormState = {
    startDate: Moment | null;
    endDate: Moment | null;
    hours: number | null;
    companyName: string;
    companyNip: string;
    city: string;
    postalCode: string;
    country: string;
    building: string;
    mentorFirstName: string;
    mentorLastName: string;
    mentorEmail: string;
    mentorPhone: string;
    kindOther: string | null;

    // relations
    kind: InternshipType | null;
    company: Company | null;
    office: BranchOffice | null;
    student: Student | null;
}

const emptyInternshipValues: InternshipFormState = {
    building: "",
    city: "",
    company: null,
    companyName: "",
    companyNip: "",
    country: "",
    endDate: null,
    hours: null,
    kind: null,
    kindOther: "",
    mentorEmail: "",
    mentorFirstName: "",
    mentorLastName: "",
    mentorPhone: "",
    office: null,
    postalCode: "",
    startDate: null,
    student: null
}

export const InternshipTypeItem = ({ type, ...props }: { type: InternshipType } & HTMLProps<any>) => {
    const info = internshipTypeLabels[type];

    return (
        <div className="internship=type-item" { ...props }>
            <div>{ info.label }</div>
            { info.description && <Typography variant="caption">{ info.description }</Typography> }
        </div>
    )
}

const InternshipProgramForm = ({ internship, onChange }: InternshipFormSectionProps) => {
    const fieldProps = formFieldProps(internship, onChange);

    const { t } = useTranslation();

    return (
        <Grid container>
            <Grid item md={ 4 }>
                <Autocomplete renderInput={ props => <TextField { ...props } label={ t("forms.internship.fields.kind") } fullWidth/> }
                              getOptionLabel={ (option: InternshipType) => internshipTypeLabels[option].label }
                              renderOption={ (option: InternshipType) => <InternshipTypeItem type={ option }/> }
                              options={ Object.values(InternshipType) as InternshipType[] }
                              disableClearable
                              { ...fieldProps("type", (event, value) => value) as any }
                />
            </Grid>
            <Grid item md={ 8 }>
                { internship.type === InternshipType.Other && <TextField label={ t("forms.internship.fields.kind") } fullWidth/> }
            </Grid>
            {/*<Grid item>*/ }
            {/*    <FormGroup>*/ }
            {/*        <FormLabel component="legend" className="subsection-header">Realizowane punkty programu praktyk (minimum 3)</FormLabel>*/ }
            {/*        { course.possibleProgramEntries.map(entry => {*/ }
            {/*            return (*/ }
            {/*                <FormControlLabel label={ entry.description } key={ entry.id }*/ }
            {/*                                  control={ <Checkbox /> }*/ }
            {/*                />*/ }
            {/*            )*/ }
            {/*        }) }*/ }
            {/*    </FormGroup>*/ }
            {/*</Grid>*/ }
        </Grid>
    )
}

const InternshipDurationForm = ({ internship, onChange }: InternshipFormSectionProps) => {
    const { t } = useTranslation();

    const [startDate, setStartDate] = useProxyState<Moment | null>(internship.startDate, value => onChange({ ...internship, startDate: value }));
    const [endDate, setEndDate] = useProxyState<Moment | null>(internship.endDate, value => onChange({ ...internship, endDate: value }));

    const [overrideHours, setHoursOverride] = useState<number | null>(null)
    const [workingHours, setWorkingHours] = useState<number>(40)

    const computedHours = useMemo(() => startDate && endDate && computeWorkingHours(startDate, endDate, workingHours / 5), [startDate, endDate, workingHours]);

    const hours = useMemo(() => overrideHours !== null ? overrideHours : computedHours || null, [overrideHours, computedHours]);
    const weeks = useMemo(() => hours !== null ? Math.floor(hours / 40) : null, [hours]);

    useEffect(() => onChange({ ...internship, hours }), [hours])

    return (
        <Grid container>
            <Grid item md={ 6 }>
                <DatePicker value={ startDate } onChange={ setStartDate }
                            format="DD MMMM yyyy"
                            clearable disableToolbar fullWidth
                            variant="inline" label={ t("forms.internship.fields.start-date") }
                            minDate={ moment() }
                />
            </Grid>
            <Grid item md={ 6 }>
                <DatePicker value={ endDate } onChange={ setEndDate }
                            format="DD MMMM yyyy"
                            clearable disableToolbar fullWidth
                            variant="inline" label={ t("forms.internship.fields.end-date") }
                            minDate={ startDate || moment() }
                />
            </Grid>
            <Grid item md={ 4 }>
                <TextField fullWidth label={ t("forms.internship.fields.working-hours") }
                           value={ workingHours } onChange={ ev => setWorkingHours(parseInt(ev.target.value) || 0) }
                           helperText={ t("forms.internship.help.working-hours") }
                />
            </Grid>
            <Grid item md={ 4 }>
                <TextField fullWidth label={ t("forms.internship.fields.total-hours") }
                           value={ hours } onChange={ ev => setHoursOverride(parseInt(ev.target.value) || 0) }
                />
            </Grid>
            <Grid item md={ 4 }>
                <TextField fullWidth label={ t("forms.internship.fields.weeks") }
                           value={ weeks } disabled
                           helperText={ t("forms.internship.help.weeks") }
                />
            </Grid>
        </Grid>
    );
}

export const InternshipForm: React.FunctionComponent<InternshipFormProps> = props => {
    const initialInternshipState = useSelector<AppState, Nullable<Internship>>(state => getInternshipProposal(state.proposal) || {
        ...emptyInternship,
        intern: sampleStudent
    });

    const [internship, setInternship] = useState<Nullable<Internship>>(initialInternshipState)
    const { t } = useTranslation();

    const dispatch = useDispatch();
    const history = useHistory();

    const [confirmDialogOpen, setConfirmDialogOpen] = useState<boolean>(false);

    const handleSubmit = () => {
        setConfirmDialogOpen(false);

        dispatch({ type: InternshipProposalActions.Send, internship: internship as Internship });
        history.push(route("home"))
    }

    const handleSubmitConfirmation = () => {
        setConfirmDialogOpen(true);
    }

    const handleCancel = () => {
        setConfirmDialogOpen(false);
    }

    const validationSchema = Yup.object<Partial<InternshipFormState>>({
        mentorFirstName: Yup.string().required(t("validation.required")),
        mentorLastName: Yup.string().required(t("validation.required")),
        mentorEmail: Yup.string()
            .required(t("validation.required"))
            .email(t("validation.email")),
        mentorPhone: Yup.string()
            .required(t("validation.required"))
            .matches(/^[+]*[(]{0,1}[0-9]{1,4}[)]{0,1}[-\s\./0-9]*$/, t("validation.phone")),
        hours: Yup.number()
            .min(40, t("validation.internship.minimum-hours")) // todo: take it from edition
    })

    return (
        <Formik initialValues={ emptyInternshipValues } onSubmit={ values => console.log(values) }
                validationSchema={ validationSchema } validateOnChange={ false } validateOnBlur={ true }>
            { formik => <Form>
                <Typography variant="h3" className="section-header">{ t('internship.sections.intern-info') }</Typography>
                <StudentForm student={ sampleStudent }/>
                <Typography variant="h3" className="section-header">{ t('internship.sections.kind' )}</Typography>
                <InternshipProgramForm internship={ internship } onChange={ setInternship }/>
                <Typography variant="h3" className="section-header">{ t('internship.sections.duration') }</Typography>
                <InternshipDurationForm internship={ internship } onChange={ setInternship }/>
                <Typography variant="h3" className="section-header">{ t('internship.sections.place') }</Typography>
                <CompanyForm internship={ internship } onChange={ setInternship }/>
                <Actions>
                    <Button variant="contained" color="primary" onClick={ () => formik.validateForm(formik.values) }>{ t("confirm") }</Button>

                    <Button component={ RouterLink } to={ route("home") }>
                        { t('go-back') }
                    </Button>
                </Actions>

                <Dialog open={ confirmDialogOpen } onClose={ handleCancel }>
                    <DialogContent>
                        <DialogContentText>{ t('forms.internship.send-confirmation') }</DialogContentText>
                    </DialogContent>
                    <DialogActions>
                        <Button onClick={ handleCancel }>{ t('cancel') }</Button>
                        <Button color="primary" autoFocus onClick={ formik.submitForm }>{ t('confirm') }</Button>
                    </DialogActions>
                </Dialog>
            </Form> }
        </Formik>
    )
}

